AbstractExpression

Enum AbstractExpression 

pub enum AbstractExpression<F: Field> {
    Constant(F),
    Selector(MleIndex<F>, Box<AbstractExpression<F>>, Box<AbstractExpression<F>>),
    Mle(NodeId),
    Sum(Box<AbstractExpression<F>>, Box<AbstractExpression<F>>),
    Product(Vec<NodeId>),
    Scaled(Box<AbstractExpression<F>>, F),
}
See [ExpressionNode] for more details. Note that this implementation is somewhat redundant with [Expression] and [ExpressionNode], but the separation allows for more flexibility with respect to this particular frontend being able to create polynomial relationships in any way it chooses, so long as those representations are compile-able into [Expression].

Variants



Constant(F)



Selector(MleIndex<F>, Box<AbstractExpression<F>>, Box<AbstractExpression<F>>)



Mle(NodeId)



Sum(Box<AbstractExpression<F>>, Box<AbstractExpression<F>>)



Product(Vec<NodeId>)



Scaled(Box<AbstractExpression<F>>, F)

Implementations

Source

impl<F: Field> AbstractExpression<F>

pub fn traverse( &self, observer_fn: &mut impl FnMut(&AbstractExpression<F>) -> Result<()>, ) -> Result<()>

Traverses the expression and applies the observer function to all nodes.

pub fn get_sources(&self) -> Vec<NodeId>

find all the sources this expression depend on

pub fn get_num_vars( &self, num_vars_map: &HashMap<NodeId, usize>, ) -> Result<usize>

Computes the num_vars of this expression (how many rounds of sumcheck it would take to prove)

pub fn build_circuit_expr( self, circuit_map: &CircuitMap, ) -> Result<Expression<F, ExprDescription>>

Convert the abstract expression into a circuit expression, which stores information on the shape of the expression, using the CircuitMap.

pub fn select(self, rhs: Self) -> Self

See documentation for select() function within [remainder::expression::circuit_expr::ExprDescription]

pub fn select_seq<E: Clone + Into<AbstractExpression<F>>>( expressions: Vec<E>, ) -> Self

Call Self::select sequentially

pub fn binary_tree_selector<E: Into<AbstractExpression<F>>>( expressions: Vec<E>, ) -> Self

Create a nested selector Expression that selects between 2^k Expressions by creating a binary tree of Selector Expressions. The order of the leaves is the order of the input expressions. (Note that this is very different from Self::select_seq.)

pub fn pow(pow: usize, node_id: Self) -> Self

Create a product Expression that raises one expression to a given power

pub fn products(node_ids: Vec<NodeId>) -> Self

Create a product Expression that multiplies many MLEs together

pub fn mult(lhs: Self, rhs: Self) -> Self

Multiplication for expressions, DO NOT USE ON SELECTORS

pub fn mle(node_id: NodeId) -> Self

Create a mle Expression that contains one MLE

pub fn constant(constant: F) -> Self

Create a constant Expression that contains one field element

pub fn negated(expression: Self) -> Self

negates an Expression

pub fn sum(lhs: Self, rhs: Self) -> Self

Create a Sum Expression that contains two MLEs

pub fn scaled(expression: AbstractExpression<F>, scale: F) -> Self

scales an Expression by a field element

Source

impl<F: Field> AbstractExpression<F>

fn build_circuit_node( self, node_map: &HashMap<NodeId, (usize, &CircuitLocation)>, ) -> Result<ExpressionNode<F, ExprDescription>>

fn get_node_ids(&self, node_ids: Vec<NodeId>) -> Vec<NodeId>